@@ -29,8 +29,9 @@ from gzip import GzipFile
 from types import UnicodeType
 
 from sqlobject import *
+from sqlobject.sqlbuilder import ISNOTNULL
 
-from imdb.parser.sql import soundex
+from imdb.parser.sql import soundex, get_movie_data
 from imdb.parser.sql.dbschema import *
 from imdb.utils import analyze_title, analyze_name, \
         build_name, build_title, normalizeName, _articles
@@ -46,7 +47,7 @@ for i, art in enumerate(_articles):
 re_nameImdbIndex = re.compile(r'\(([IVXLCDM]+)\)')
 
 HELP = """imdbpy2sql.py usage:
-    %s -d /directory/with/PlainTextDataFiles/ -u URI
+    %s -d /directory/with/PlainTextDataFiles/ -u URI [--COMPATIBILITY-OPTIONS]
 
         # NOTE: URI is something along the line:
                 scheme://[user[:password]@]host[:port]/database[?parameters]
@@ -57,6 +58,14 @@ HELP = """imdbpy2sql.py usage:
                 sqlite:/tmp/imdb.db
                 sqlite:/C|/full/path/to/database
 
+        # NOTE: --COMPATIBILITY-OPTIONS can be one of:
+            --mysql-innodb          insert data into a MySQL MyISAM db,
+                                    and then convert it to InnoDB.
+            --ms-sqlserver          compatibility mode for Microsoft SQL Server
+                                    and SQL Express.
+            --sqlite-transactions   uses transactions, to speed-up SQLite.
+
+
                 See README.sqldb for more information.
 """ % sys.argv[0]
 
@@ -64,11 +73,41 @@ HELP = """imdbpy2sql.py usage:
 IMDB_PTDF_DIR = None
 # URI used to connect to the database.
 URI = None
+# Store custom queries specified on the command line.
+CUSTOM_QUERIES = {}
+# Allowed time specification, for custom queries.
+ALLOWED_TIMES = ('BEGIN', 'BEFORE_DROP', 'BEFORE_CREATE', 'AFTER_CREATE',
+                'BEFORE_MOVIES', 'BEFORE_CAST', 'BEFORE_RESTORE',
+                'BEFORE_INDEXES', 'END', 'BEFORE_MOVIES_TODB',
+                'AFTER_MOVIES_TODB', 'BEFORE_PERSONS_TODB',
+                'AFTER_PERSONS_TODB','BEFORE_SQLDATA_TODB',
+                'AFTER_SQLDATA_TODB', 'BEFORE_AKAMOVIES_TODB',
+                'AFTER_AKAMOVIES_TODB', 'BEFORE_EVERY_TODB',
+                'AFTER_EVERY_TODB')
+
+# Shortcuts for some compatibility options.
+MYSQLINNODB_OPTS = ['-e',
+        'AFTER_CREATE:FOR_EVERY_TABLE:ALTER TABLE %(table)s ENGINE=MyISAM;',
+        '-e', 'END:FOR_EVERY_TABLE:ALTER TABLE %(table)s ENGINE=InnoDB;']
+SQLSERVER_OPTS = ['-e', 'BEFORE_EVERY_TODB:SET IDENTITY_INSERT %(table)s ON;',
+        '-e', 'AFTER_EVERY_TODB:SET IDENTITY_INSERT %(table)s OFF;']
+SQLITE_OPTS = ['-e', 'BEFORE_EVERY_TODB:BEGIN TRANSACTION;',
+        '-e', 'AFTER_EVERY_TODB:COMMIT;']
+
+if '--mysql-innodb' in sys.argv[1:]:
+    sys.argv += MYSQLINNODB_OPTS
+if '--ms-sqlserver' in sys.argv[1:]:
+    sys.argv += SQLSERVER_OPTS
+if '--sqlite-transactions' in sys.argv[1:]:
+    sys.argv += SQLITE_OPTS
 
 # Manage arguments list.
 try:
-    optlist, args = getopt.getopt(sys.argv[1:], 'u:d:h',
-                                                ['uri=', 'data=', 'help'])
+    optlist, args = getopt.getopt(sys.argv[1:], 'u:d:e:h',
+                                                ['uri=', 'data=', 'execute=',
+                                                'mysql-innodb', 'ms-sqlserver',
+                                                'sqlite-transactions',
+                                                'help'])
 except getopt.error, e:
     print 'Troubles with arguments.'
     print HELP
@@ -79,6 +118,24 @@ for opt in optlist:
         IMDB_PTDF_DIR = opt[1]
     elif opt[0] in ('-u', '--uri'):
         URI = opt[1]
+    elif opt[0] in ('-e', '--execute'):
+        if opt[1].find(':') == -1:
+            print 'WARNING: wrong command syntax: "%s"' % opt[1]
+            continue
+        when, cmd = opt[1].split(':', 1)
+        if when not in ALLOWED_TIMES:
+            print 'WARNING: unknown time: "%s"' % when
+            continue
+        if when == 'BEFORE_EVERY_TODB':
+            for nw in ('BEFORE_MOVIES_TODB', 'BEFORE_PERSONS_TODB',
+                        'BEFORE_SQLDATA_TODB', 'BEFORE_AKAMOVIES_TODB'):
+                CUSTOM_QUERIES.setdefault(nw, []).append(cmd)
+        elif when == 'AFTER_EVERY_TODB':
+            for nw in ('AFTER_MOVIES_TODB', 'AFTER_PERSONS_TODB',
+                        'AFTER_SQLDATA_TODB', 'AFTER_AKAMOVIES_TODB'):
+                CUSTOM_QUERIES.setdefault(nw, []).append(cmd)
+        else:
+            CUSTOM_QUERIES.setdefault(when, []).append(cmd)
     elif opt[0] in ('-h', '--help'):
         print HELP
         sys.exit(0)
@@ -407,6 +464,8 @@ class _BaseCache(dict):
         self._flushing = 0
         self._deferredData = {}
         self._recursionLevel = 0
+        self._table_name = ''
+        self._id_for_custom_q = ''
         if d is not None:
             for k, v in d.iteritems(): self[k] = v
 
@@ -432,10 +491,23 @@ class _BaseCache(dict):
             self._tmpDict.clear()
             return
         if self._tmpDict:
+            # Horrible hack to know if AFTER_%s_TODB has run.
+            _after_has_run = False
+            keys = {'table': self._table_name}
             try:
+                executeCustomQueries('BEFORE_%s_TODB' % self._id_for_custom_q,
+                                    _keys=keys, _timeit=False)
                 self._toDB(quiet)
+                executeCustomQueries('AFTER_%s_TODB' % self._id_for_custom_q,
+                                    _keys=keys, _timeit=False)
+                _after_has_run = True
                 self._tmpDict.clear()
             except OperationalError, e:
+                # XXX: I'm not sure this is the right thing (and way)
+                #      to proceed.
+                if not _after_has_run:
+                    executeCustomQueries('AFTER_%s_TODB'%self._id_for_custom_q,
+                                        _keys=keys, _timeit=False)
                 # Dataset too large; split it in two and retry.
                 # XXX: new code!
                 # the same class instance (self) is used, instead of
@@ -497,10 +569,11 @@ class _BaseCache(dict):
 def fetchsome(curs, size=20000):
     """Yes, I've read the Python Cookbook! :-)"""
     while 1:
-        res = CURS.fetchmany(size)
+        res = curs.fetchmany(size)
         if not res: break
         for r in res: yield r
 
+
 class MoviesCache(_BaseCache):
     """Manage the movies list."""
     className = 'MoviesCache'
@@ -509,6 +582,8 @@ class MoviesCache(_BaseCache):
     def __init__(self, *args, **kwds):
         _BaseCache.__init__(self, *args, **kwds)
         self.movieYear = {}
+        self._table_name = tableName(Title)
+        self._id_for_custom_q = 'MOVIES'
         self.sqlstr, self.converter = createSQLstr(Title, ('id', 'title',
                                     'imdbIndex', 'kindID', 'productionYear',
                                     'phoneticCode', 'episodeOfID',
@@ -547,7 +622,8 @@ class MoviesCache(_BaseCache):
                 if series_d['year'] is None: del series_d['year']
                 else: series_d['year'] = str(series_d['year'])
                 mdict['episode of'] = series_d
-            title = build_title(mdict, canonical=1, ptdf=1)
+            title = build_title(mdict, canonical=1, ptdf=1,
+                                _emptyString='')
             dict.__setitem__(self, title, x[0])
         self.counter = counter(Title.select().count() + 1)
         Title.sqlmeta.cacheValues = _oldcacheValues
@@ -572,7 +648,8 @@ class MoviesCache(_BaseCache):
             kind = tget('kind')
             if kind == 'episode':
                 #series title
-                stitle = build_title(tget('episode of'), canonical=1)
+                stitle = build_title(tget('episode of'), canonical=1,
+                                    _emptyString='')
                 episodeOf = self.addUnique(stitle)
                 del t['episode of']
                 year = self.movieYear.get(v)
@@ -598,6 +675,8 @@ class PersonsCache(_BaseCache):
 
     def __init__(self, *args, **kwds):
         _BaseCache.__init__(self, *args, **kwds)
+        self._table_name = tableName(Name)
+        self._id_for_custom_q = 'PERSONS'
         self.sqlstr, self.converter = createSQLstr(Name, ['id', 'name',
                                 'imdbIndex', 'namePcodeCf', 'namePcodeNf',
                                 'surnamePcode'])
@@ -660,6 +739,7 @@ class SQLData(dict):
         self.sqlString = sqlString
         self.converter = converter
         self._recursionLevel = 1
+        self._table_name = tableName(table)
         for k, v in d.items(): self[k] = v
 
     def __setitem__(self, key, value):
@@ -688,11 +768,21 @@ class SQLData(dict):
             self.clear()
             self.counter = self.counterInit
             return
+        keys = {'table': self._table_name}
+        _after_has_run = False
         try:
+            executeCustomQueries('BEFORE_SQLDATA_TODB', _keys=keys,
+                                _timeit=False)
             self._toDB()
+            executeCustomQueries('AFTER_SQLDATA_TODB', _keys=keys,
+                                _timeit=False)
+            _after_has_run = True
             self.clear()
             self.counter = self.counterInit
         except OperationalError, e:
+            if not _after_has_run:
+                executeCustomQueries('AFTER_SQLDATA_TODB', _keys=keys,
+                                    _timeit=False)
             print ' * TOO MANY DATA (%s items), SPLITTING (run #%d)...' % \
                     (len(self), self._recursionLevel)
             self._recursionLevel += 1
@@ -822,7 +912,11 @@ def doCast(fp, roleid, rolename):
         for item in ll[1:]:
             if not item: continue
             if item[0] == '[':
-                role = item[1:-1]
+                # Quite inefficient, but there are some very strange
+                # cases of garbage in the plain text data files to handle...
+                role = item[1:]
+                if role[-1:] == ']':
+                    role = role[:-1]
                 if role[-1:] == ')':
                     nidx = role.find('(')
                     if nidx != -1:
@@ -919,6 +1013,8 @@ class AkasMoviesCache(MoviesCache):
         self.flushEvery = 50000
         self.notes = {}
         self.ids = {}
+        self._table_name = tableName(AkaTitle)
+        self._id_for_custom_q = 'AKAMOVIES'
         self.sqlstr, self.converter = createSQLstr(AkaTitle, ('id', 'movieID',
                             'title', 'imdbIndex', 'kindID', 'productionYear',
                             'phoneticCode', 'episodeOfID', 'seasonNr',
@@ -1209,7 +1305,15 @@ def nmmvFiles(fp, funct, fname):
                         title = i.get('long imdb canonical title')
                         if not title: continue
                         movieid = CACHE_MID.addUnique(title)
-                        guestdata.add((mopid, movieid, i.currentRole or None,
+                        crole = i.currentRole
+                        if isinstance(crole, list):
+                            crole = ' / '.join([x.get('long imdb name', u'')
+                                                for x in crole])
+                        if not crole:
+                            crole = None
+                        else:
+                            crole = unicode(crole).encode('utf_8')
+                        guestdata.add((mopid, movieid, crole,
                                         i.notes or None))
                         continue
                     if k in ('plot', 'mini biography'):
@@ -1364,7 +1468,7 @@ def getTopBottomRating():
             mid = CACHE_MID.addUnique(title)
             if what == 'top 250 rank': rank = count
             else: rank = 11 - count
-            sqldata.add((mid, rank))
+            sqldata.add((mid, str(rank)))
             count += 1
         sqldata.flush()
         fp.close()
@@ -1423,7 +1527,6 @@ CACHE_MID = MoviesCache()
 CACHE_PID = PersonsCache()
 
 
-
 def _cmpfunc(x, y):
     """Sort a list of tuples, by the length of the first item (in reverse)."""
     lx = len(x[0])
@@ -1457,22 +1560,147 @@ def readConstants():
         CCAST_TYPES[x.kind] = x.id
 
 
+def notNULLimdbID(cls):
+    """Return a list of dictionaries for titles or names for which a
+    imdbID is present in the database."""
+    if cls is Title: cname = 'movies'
+    else: cname = 'people'
+    print 'SAVING imdbID values for %s...' % cname,
+    sys.stdout.flush()
+    try:
+        tons = cls.select(ISNOTNULL(cls.q.imdbID))
+    except:
+        print 'SKIPPING: no data.'
+        return []
+    results = []
+    _kdict = {}
+    try:
+        for x in KindType.select():
+            _kdict[x.id] = x.kind
+    except:
+        print 'SKIPPING: no data.'
+        return []
+    for t in tons:
+        if cls is Title:
+            md = get_movie_data(t.id, _kdict)
+        else:
+            md = {'name': t.name}
+            if t.imdbIndex is not None:
+                md['imdbIndex'] = t.imdbIndex
+        md['imdbID'] = t.imdbID
+        results.append(md)
+    print 'DONE! (%d entries)' % len(results)
+    return results
+
+
+def restoreImdbID(tons, cls):
+    """Restore imdbID for movies or people."""
+    if cls is Title:
+        CACHE = CACHE_MID
+        cname = 'movies'
+    else:
+        CACHE = CACHE_PID
+        cname = 'people'
+    print 'RESTORING imdbID values for %s...' % cname,
+    sys.stdout.flush()
+    count = 0
+    for t in tons:
+        if cls is Title:
+            t_str = build_title(t, canonical=1, ptdf=1)
+        else:
+            t_str = build_name(t, canonical=1)
+        t_str = t_str.encode('utf_8')
+        db_mopID = CACHE.get(t_str)
+        if db_mopID is None:
+            continue
+        try:
+            mop_in_db = cls.get(db_mopID)
+            try:
+                mop_in_db.imdbID = t['imdbID']
+            except:
+                continue
+        except SQLObjectNotFound:
+            continue
+        count += 1
+    print 'DONE! (restored %d entries out of %d)' % (count, len(tons))
+
+
+def _executeQuery(query):
+    print 'EXECUTING "%s"...' % (query),
+    sys.stdout.flush()
+    try:
+        CURS.execute(query)
+        print 'DONE!'
+    except Exception, e:
+        print 'FAILED (%s)!' % e
+
+
+def executeCustomQueries(when, _keys=None, _timeit=True):
+    """Run custom queries as specified on the command line."""
+    if _keys is None: _keys = {}
+    for query in CUSTOM_QUERIES.get(when, []):
+        print 'EXECUTING "%s:%s"...' % (when, query)
+        sys.stdout.flush()
+        if query.startswith('FOR_EVERY_TABLE:'):
+            query = query[16:]
+            CURS.execute('SHOW TABLES;')
+            tables = [x[0] for x in CURS.fetchall()]
+            for table in tables:
+                try:
+                    keys = {'table': table}
+                    keys.update(_keys)
+                    _executeQuery(query % keys)
+                    if _timeit:
+                        t('%s command' % when)
+                except Exception, e:
+                    print 'FAILED (%s)!' % e
+        else:
+            _executeQuery(query % _keys)
+            if _timeit:
+                t('%s command' % when)
+
+
 # begin the iterations...
 def run():
     print 'RUNNING imdbpy2sql.py'
+
+    executeCustomQueries('BEGIN')
+
+    # Storing imdbIDs for movies and persons.
+    try:
+        movies_imdbIDs = notNULLimdbID(Title)
+    except:
+        movies_imdbIDs = []
+        print 'WARNING: failed to read imdbIDs for movies'
+    try:
+        people_imdbIDs = notNULLimdbID(Name)
+    except:
+        people_imdbIDs = []
+        print 'WARNING: failed to read imdbIDs for people'
+
+    executeCustomQueries('BEFORE_DROP')
+
     # Truncate the current database.
     print 'DROPPING current database...',
     sys.stdout.flush()
     dropTables()
-    print 'done!'
+    print 'DONE!'
+
+    executeCustomQueries('BEFORE_CREATE')
+
     # Rebuild the database structure.
     print 'CREATING new tables...',
     sys.stdout.flush()
     createTables()
+    print 'DONE!'
+    t('dropping and recreating the database')
+
+    executeCustomQueries('AFTER_CREATE')
+
     # Read the constants.
     readConstants()
-    print 'done!'
-    t('dropping and recreating the database')
+
+    executeCustomQueries('BEFORE_CAST')
 
     # Populate the CACHE_MID instance.
     readMovieList()
@@ -1522,6 +1750,20 @@ def run():
     completeCast()
     t('completeCast()')
 
+    executeCustomQueries('BEFORE_RESTORE')
+
+    # Restoring imdbIDs for movies and persons.
+    try:
+        restoreImdbID(movies_imdbIDs, Title)
+        del movies_imdbIDs
+    except:
+        print 'WARNING: failed to restore imdbIDs for movies'
+    try:
+        restoreImdbID(people_imdbIDs, Name)
+        del people_imdbIDs
+    except:
+        print 'WARNING: failed to restore imdbIDs for people'
+
     # Flush caches.
     CACHE_MID.flush()
     CACHE_MID.clear()
@@ -1532,12 +1774,16 @@ def run():
     print 'TOTAL TIME TO INSERT DATA: %d minutes, %d seconds' % \
             divmod(int(time.time())-BEGIN_TIME, 60)
 
+    executeCustomQueries('BEFORE_INDEXES')
+
     print 'building database indexes (this may take a while)'
     sys.stdout.flush()
     # Build database indexes.
     createIndexes()
     t('createIndexes()')
 
+    executeCustomQueries('END')
+
     print 'DONE! (in %d minutes, %d seconds)' % \
             divmod(int(time.time())-BEGIN_TIME, 60)
 
